#!/usr/bin/env bash

# DESCRIPTION
#
# - create a dart ffi plugin under guest research directory
# - generate boilerplate files
# - fill in and install basic dependencies

Quit_On_Error() {
	if [ "$?" -ne "0" ]; then
		echo "**** BUILD FAILED ****"
		exit 1;
	fi
}


PROJ_NAME=${1?"Usage: $0 ProjName Overwrite{yes|no}"}
OVERWRITE=${2?"Usage: $0 ProjName Overwrite{yes|no}"}
OptOverwrite=
if [[ "$OVERWRITE" == "yes" ]]; then
	OptOverwrite="--overwrite"
fi

ScriptDir="$( cd "$( dirname "$0" )" && pwd )"
cd - &> /dev/null

RootDir="$ScriptDir"/..
GuestResearchDir="$RootDir/research/hello-mamguest"

mkdir -p "$GuestResearchDir" &> /dev/null

pushd .

# create plugin proj

cd "$GuestResearchDir"
# CAUTION
# - flutter 1.20: must add comma-separated platforms to support multiplatform yaml
flutter create --template=plugin --platforms android,ios $OptOverwrite "${PROJ_NAME}"
Quit_On_Error

cd "${PROJ_NAME}"
PluginDir=`pwd`

# generate a default c++ source file for testing

cat > lib/"${PROJ_NAME}".cpp << EOF

#include <cstdlib>
#include <ctime>
#include <chrono>
#include <condition_variable>
#include <deque>
#include <mutex>
#include <thread>

#include <unistd.h>

#ifdef __cplusplus
    #define EXTERNC extern "C" __attribute__((visibility("default"))) __attribute__((used))
#else
    #define EXTERNC
#endif  // #ifdef __cplusplus


EXTERNC int32_t native_add(int32_t x, int32_t y) {
    return x + y;
}

EOF
Quit_On_Error

# generate cmakelists

AndroidSrcDir=$PluginDir/example/android/app/src/main/cpp
mkdir -p "${AndroidSrcDir}" &> /dev/null

cat > "$AndroidSrcDir"/CMakeLists.txt << EOF

cmake_minimum_required(VERSION 3.4.1)

project("${PROJ_NAME}")

set(PLUGIN_ROOT \${CMAKE_CURRENT_SOURCE_DIR}/../../../../../..)
set(LIB_DIR \${PLUGIN_ROOT}/lib)
set(EXAMPLE_DIR \${PLUGIN_ROOT}/example)


add_library( ${PROJ_NAME}
    # Sets the library as a shared library.
    SHARED

    # Provides a relative path to your source file(s).
    \${LIB_DIR}/${PROJ_NAME}.cpp
)

EOF
Quit_On_Error

# insert cmake as gradle dep

GradleFile="$PluginDir"/example/android/app/build.gradle

sed -i '' "55 i\\
\    \\
\    externalNativeBuild {\\
\        cmake {\\
\            path \"src/main/cpp/CMakeLists.txt\"\\
\        }\\
\    }\\
" "$GradleFile"
Quit_On_Error

# insert lib loading code to dart

DartLibFile="$PluginDir/lib/${PROJ_NAME}.dart"

sed -i '' "5 i\\
\ \\
\final int Function(int x, int y) nativeAdd =\\
\  nativeLib\\
\    .lookup<NativeFunction<Int32 Function(Int32, Int32)>>(\"native_add\")\\
\    .asFunction();\\
\ \\
" "$DartLibFile"
Quit_On_Error

sed -i '' "5 i\\
\ \\
\import 'dart:io' show Platform; // For Platform.isX\\
\import 'dart:ffi'; // For FFI\\
\import 'package:ffi/ffi.dart';\\
\ \\
\ \\
\final DynamicLibrary nativeLib = Platform.isAndroid\\
\   ? DynamicLibrary.open(\"lib${PROJ_NAME}.so\")\\
\   : DynamicLibrary.process();\\
\ \\
" "$DartLibFile"
Quit_On_Error

# replace main with ffi function

DartMainFile="$PluginDir/example/lib/main.dart"

sed -i '' "53s/.*/\ \ \ \ \ \ \ \ \ \ child:\ Text\(\'Running\ on:\ \$_platformVersion\\\n1\ \+\ 2\ ==\ \${nativeAdd\(1,\ 2\)}\'\),/" "$DartMainFile"
Quit_On_Error

# add dep: ffi package

YamlFile="$PluginDir"/pubspec.yaml

sed -i '' "6 i\\
\ \\
\import 'dart:ffi'; // For FFI\\
\import 'package:ffi/ffi.dart';\\
\ \\
" "$DartMainFile"
Quit_On_Error

sed -i '' "14 i\\
\  ffi: ^0.1.3\\
\ \\
" "$YamlFile"
Quit_On_Error

flutter pub get
Quit_On_Error

# ios: fully generate cocoapods dependency

cd example
flutter build ios

popd &> /dev/null


echo "DONE: flutter ffi plugin created: $PluginDir"
echo "TODO:"
echo "- run flutter project once targetting ios device to trigger pod install"
echo "- open iOS module in Xcode"
echo "- update provisioning"
echo "- add lib/${PROJ_NAME}.cpp to Xcode project"
